{% load i18n %}
{% load humanize %}

<div role="tabpanel" class="tab-pane active" id="tab-current_month">
    <div class="card-body bg-secondary table-responsive">
        <select class="form-select" style="width: 130px; display: none;" id="monthDropdown">
            <option value="1">{% trans "January" %}</option>
            <option value="2">{% trans "February" %}</option>
            <option value="3">{% trans "March" %}</option>
            <option value="4">{% trans "April" %}</option>
            <option value="5">{% trans "May" %}</option>
            <option value="6">{% trans "June" %}</option>
            <option value="7">{% trans "July" %}</option>
            <option value="8">{% trans "August" %}</option>
            <option value="9">{% trans "September" %}</option>
            <option value="10">{% trans "October" %}</option>
            <option value="11">{% trans "November" %}</option>
            <option value="12">{% trans "December" %}</option>
        </select>
        <br>

        <table class="table table-striped table-hover" id="ratting" style="width: 100%;">
            <br>
            <thead>
                <th class="col-main-character" style="width: 20%;">Main Character</th>
                <th class="col-total-amount" style="width: 20%;">Ratting Amount</th>
                <th class="col-total-ess" style="width: 20%;">ESS Payout</th>
                <th class="col-total-mining" style="width: 15%;">Mining Amount</th>
                <th class="col-total-others" style="width: 30%;">Misc. Amount</th>
                <th class="col-total-action" style="width: 1%;"></th>
            </thead>

            <tbody>
            </tbody>

            <tfoot id="foot">
                <tr>
                    <th><strong>{% translate "Summary" %}</strong></th>
                    <th class="col-total-amount"></th>
                    <th class="col-total-ess"></th>
                    <th class="col-total-mining"></th>
                    <th class="col-total-others"></th>
                    <th></th>
                </tr>
                <tr>
                    <th><strong>{% translate "Costs" %}</strong></th>
                    <th></th>
                    <th></th>
                    <th></th>
                    <th class="col-total-costs"></th>
                    <th></th>
                </tr>
                <tr>
                    <th><strong>{% translate "Total" %}</strong></th>
                    <th></th>
                    <th></th>
                    <th></th>
                    <th class="col-total-gesamt"></th>
                    <th class="col-total-button"></th>
                </tr>
            </tfoot>
        </table>
    </div>
    <br>
    <div class="rounded-top">
        <div class="row g-0">
            <div class="col-md-6 g-2" id="rattingBarContainer">
                <div class="card-header bg-primary">
                    <h3 class="card-title">Ledger 30 Days</h3>
                </div>
                <div class="card-body bg-secondary">
                    <div id="rattingBar"></div>
                </div>
            </div>
            <div class="col-md-6 g-2" id="walletChartContainer">
                <div class="card-header bg-primary">
                    <h3 class="card-title">Wallet</h3>
                </div>
                <div class="card-body bg-secondary">
                    <div id="walletChart"></div>
                </div>
            </div>
            <div class="col-md-6 g-2" id="workGaugeContainer">
                <div class="card-header bg-primary">
                    <h3 class="card-title">Workflow</h3>
                </div>
                <div class="card-body bg-secondary">
                    <div id="workGauge"></div>
                </div>
            </div>
        </div>
    </div>

</div>
