{% load i18n %}
{% load humanize %}

<div role="tabpanel" class="tab-pane" id="tab-all_month">
    <div class="card-body bg-secondary table-responsive">
        <table class="table table-striped table-hover" id="ratting_year" style="width: 100%;">
            <br>
            <thead>
                <th class="col-main-character" style="width: 20%;">Main Character</th>
                <th class="col-total-amount" style="width: 20%;">Ratting Amount</th>
                <th class="col-total-ess" style="width: 20%;">ESS Payout</th>
                <th class="col-total-mining" style="width: 15%;">Mining Amount</th>
                <th class="col-total-others" style="width: 30%;">Misc. Amount</th>
                <th class="col-total-action" style="width: 1%;"></th>
            </thead>

            <tbody>
            </tbody>

            <tfoot id="foot-year">
                <tr>
                    <th><strong>{% translate "Summary" %}</strong></th>
                    <th class="col-total-amount"></th>
                    <th class="col-total-ess"></th>
                    <th class="col-total-mining"></th>
                    <th class="col-total-others"></th>
                    <th></th>
                </tr>
                <tr>
                    <th><strong>{% translate "Costs" %}</strong></th>
                    <th></th>
                    <th></th>
                    <th></th>
                    <th class="col-total-costs"></th>
                    <th></th>
                </tr>
                <tr>
                    <th><strong>{% translate "Total" %}</strong></th>
                    <th></th>
                    <th></th>
                    <th></th>
                    <th class="col-total-gesamt"></th>
                    <th class="col-total-button"></th>
                </tr>
            </tfoot>
        </table>
    </div>
    <br>
    <div class="rounded-top">
        <div class="row g-0">
            <div class="col-md-6 g-2" id="rattingBarContainerYear">
                <div class="card-header bg-primary">
                    <h3 class="card-title">Ledger Year</h3>
                </div>
                <div class="card-body bg-secondary">
                    <div id="rattingBarYear"></div>
                </div>
            </div>
            <div class="col-md-6 g-2" id="walletChartContainerYear">
                <div class="card-header bg-primary">
                    <h3 class="card-title">Wallet Year</h3>
                </div>
                <div class="card-body bg-secondary">
                    <div id="walletChartYear"></div>
                </div>
            </div>
            <div class="col-md-6 g-2" id="workGaugeContainerYear">
                <div class="card-header bg-primary">
                    <h3 class="card-title">Workflow Year</h3>
                </div>
                <div class="card-body bg-secondary">
                    <div id="workGaugeYear"></div>
                </div>
            </div>
        </div>
    </div>

</div>
